#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <assert.h>
#include "vk_enum_to_str.h"
#include "vk_instance.h"
#include "vk_device.h"

bool
vk_create_instance(struct vk_device *dev)
{
   struct vk_instance_info *instance_info = &dev->instance_info;

   /* reserve one slot for MoltenVK */
   const char *layers[1] = {0};
   uint32_t num_layers = 0;
   
   const char *extensions[11] = {0};
   uint32_t num_extensions = 0;

   bool have_EXT_debug_utils = false;
   bool have_KHR_get_physical_device_properties2 = false;
   bool have_KHR_external_memory_capabilities = false;
   bool have_KHR_external_semaphore_capabilities = false;
   bool have_MVK_moltenvk = false;
   bool have_KHR_surface = false;
   bool have_EXT_headless_surface = false;
   bool have_KHR_wayland_surface = false;
   bool have_KHR_xcb_surface = false;
   bool have_KHR_win32_surface = false;


#if defined(MVK_VERSION)
   bool have_moltenvk_layer = false;
#endif

   GET_PROC_ADDR_INSTANCE_LOCAL(dev, NULL, EnumerateInstanceExtensionProperties);
   GET_PROC_ADDR_INSTANCE_LOCAL(dev, NULL, EnumerateInstanceLayerProperties);
   if (!vk_EnumerateInstanceExtensionProperties ||
       !vk_EnumerateInstanceLayerProperties)
      return false;

   // Build up the extensions from the reported ones but only for the unnamed layer
   uint32_t extension_count = 0;
   if (vk_EnumerateInstanceExtensionProperties(NULL, &extension_count, NULL) != VK_SUCCESS) {
       fprintf(stderr, "VK: vkEnumerateInstanceExtensionProperties failed\n");
   } else {
       VkExtensionProperties *extension_props = malloc(extension_count * sizeof(VkExtensionProperties));
       if (extension_props) {
           if (vk_EnumerateInstanceExtensionProperties(NULL, &extension_count, extension_props) != VK_SUCCESS) {
              fprintf(stderr, "VK: vkEnumerateInstanceExtensionProperties failed\n");
           } else {
              for (uint32_t i = 0; i < extension_count; i++) {
                if (!strcmp(extension_props[i].extensionName, "VK_EXT_debug_utils")) {
                    have_EXT_debug_utils = true;
                }
                if (!strcmp(extension_props[i].extensionName, "VK_KHR_get_physical_device_properties2")) {
                    have_KHR_get_physical_device_properties2 = true;
                }
                if (!strcmp(extension_props[i].extensionName, "VK_KHR_external_memory_capabilities")) {
                    have_KHR_external_memory_capabilities = true;
                }
                if (!strcmp(extension_props[i].extensionName, "VK_KHR_external_semaphore_capabilities")) {
                    have_KHR_external_semaphore_capabilities = true;
                }
                if (!strcmp(extension_props[i].extensionName, "VK_MVK_moltenvk")) {
                    have_MVK_moltenvk = true;
                }
                if (!strcmp(extension_props[i].extensionName, "VK_KHR_surface")) {
                    have_KHR_surface = true;
                }
                if (!strcmp(extension_props[i].extensionName, "VK_EXT_headless_surface")) {
                    have_EXT_headless_surface = true;
                }
                if (!strcmp(extension_props[i].extensionName, "VK_KHR_wayland_surface")) {
                    have_KHR_wayland_surface = true;
                }
                if (!strcmp(extension_props[i].extensionName, "VK_KHR_xcb_surface")) {
                    have_KHR_xcb_surface = true;
                }
                if (!strcmp(extension_props[i].extensionName, "VK_KHR_win32_surface")) {
                    have_KHR_win32_surface = true;
                }
              }
           }
       free(extension_props);
       }
   }

    // Build up the layers from the reported ones
    uint32_t layer_count = 0;

    if (vk_EnumerateInstanceLayerProperties(&layer_count, NULL) != VK_SUCCESS) {
        fprintf(stderr, "VK: vkEnumerateInstanceLayerProperties failed\n");
    } else {
        VkLayerProperties *layer_props = malloc(layer_count * sizeof(VkLayerProperties));
        if (layer_props) {
            if (vk_EnumerateInstanceLayerProperties(&layer_count, layer_props) != VK_SUCCESS) {
                fprintf(stderr, "VK: vkEnumerateInstanceLayerProperties failed\n");
            } else {
               for (uint32_t i = 0; i < layer_count; i++) {
#if defined(MVK_VERSION)
                  if (!strcmp(layer_props[i].layerName, "MoltenVK")) {
                     have_moltenvk_layer = true;
                     layers[num_layers++] = "MoltenVK";
                  }
#endif
               }
            }
        free(layer_props);
        }
    }

   if (have_EXT_debug_utils ) {
      instance_info->have_EXT_debug_utils = have_EXT_debug_utils;
      extensions[num_extensions++] = "VK_EXT_debug_utils";
   }
   if (have_KHR_get_physical_device_properties2 ) {
      instance_info->have_KHR_get_physical_device_properties2 = have_KHR_get_physical_device_properties2;
      extensions[num_extensions++] = "VK_KHR_get_physical_device_properties2";
   }
   if (have_KHR_external_memory_capabilities ) {
      instance_info->have_KHR_external_memory_capabilities = have_KHR_external_memory_capabilities;
      extensions[num_extensions++] = "VK_KHR_external_memory_capabilities";
   }
   if (have_KHR_external_semaphore_capabilities ) {
      instance_info->have_KHR_external_semaphore_capabilities = have_KHR_external_semaphore_capabilities;
      extensions[num_extensions++] = "VK_KHR_external_semaphore_capabilities";
   }
   if (have_MVK_moltenvk ) {
      instance_info->have_MVK_moltenvk = have_MVK_moltenvk;
      extensions[num_extensions++] = "VK_MVK_moltenvk";
   }
   if (have_KHR_surface ) {
      instance_info->have_KHR_surface = have_KHR_surface;
      extensions[num_extensions++] = "VK_KHR_surface";
   }
   if (have_EXT_headless_surface ) {
      instance_info->have_EXT_headless_surface = have_EXT_headless_surface;
      extensions[num_extensions++] = "VK_EXT_headless_surface";
   }
   if (have_KHR_wayland_surface ) {
      instance_info->have_KHR_wayland_surface = have_KHR_wayland_surface;
      extensions[num_extensions++] = "VK_KHR_wayland_surface";
   }
   if (have_KHR_xcb_surface && (!instance_info->disable_xcb_surface)) {
      instance_info->have_KHR_xcb_surface = have_KHR_xcb_surface;
      extensions[num_extensions++] = "VK_KHR_xcb_surface";
   }
   if (have_KHR_win32_surface ) {
      instance_info->have_KHR_win32_surface = have_KHR_win32_surface;
      extensions[num_extensions++] = "VK_KHR_win32_surface";
   }


   VkApplicationInfo ai = {0};
   ai.sType = VK_STRUCTURE_TYPE_APPLICATION_INFO;

   ai.pApplicationName = "vkoverhead";

   ai.pEngineName = "vkoverhead";
   ai.apiVersion = instance_info->loader_version;

   VkInstanceCreateInfo ici = {0};
   ici.sType = VK_STRUCTURE_TYPE_INSTANCE_CREATE_INFO;
   ici.pApplicationInfo = &ai;
   ici.ppEnabledExtensionNames = extensions;
   ici.enabledExtensionCount = num_extensions;
   ici.ppEnabledLayerNames = layers;
   ici.enabledLayerCount = num_layers;

   GET_PROC_ADDR_INSTANCE_LOCAL(dev, NULL, CreateInstance);
   assert(vk_CreateInstance);

   VkResult err = vk_CreateInstance(&ici, NULL, &dev->instance);
   if (err != VK_SUCCESS) {
      fprintf(stderr, "VK: vkCreateInstance failed (%s)\n", vk_Result_to_str(err));
      return false;
   }

   return true;
}

void
vk_verify_instance_extensions(struct vk_device *dev)
{
   if (dev->instance_info.have_EXT_debug_utils) {
      if (!dev->vk.CreateDebugUtilsMessengerEXT) {
#ifndef NDEBUG
         dev->vk.CreateDebugUtilsMessengerEXT = (PFN_vkCreateDebugUtilsMessengerEXT)vk_stub_CreateDebugUtilsMessengerEXT;
#else
         dev->vk.CreateDebugUtilsMessengerEXT = (PFN_vkCreateDebugUtilsMessengerEXT)vk_stub_function_not_loaded;
#endif
      }
      if (!dev->vk.DestroyDebugUtilsMessengerEXT) {
#ifndef NDEBUG
         dev->vk.DestroyDebugUtilsMessengerEXT = (PFN_vkDestroyDebugUtilsMessengerEXT)vk_stub_DestroyDebugUtilsMessengerEXT;
#else
         dev->vk.DestroyDebugUtilsMessengerEXT = (PFN_vkDestroyDebugUtilsMessengerEXT)vk_stub_function_not_loaded;
#endif
      }
      if (!dev->vk.SubmitDebugUtilsMessageEXT) {
#ifndef NDEBUG
         dev->vk.SubmitDebugUtilsMessageEXT = (PFN_vkSubmitDebugUtilsMessageEXT)vk_stub_SubmitDebugUtilsMessageEXT;
#else
         dev->vk.SubmitDebugUtilsMessageEXT = (PFN_vkSubmitDebugUtilsMessageEXT)vk_stub_function_not_loaded;
#endif
      }
   }
   if (dev->instance_info.have_KHR_get_physical_device_properties2) {
      if (!dev->vk.GetPhysicalDeviceFeatures2KHR) {
#ifndef NDEBUG
         dev->vk.GetPhysicalDeviceFeatures2KHR = (PFN_vkGetPhysicalDeviceFeatures2KHR)vk_stub_GetPhysicalDeviceFeatures2KHR;
#else
         dev->vk.GetPhysicalDeviceFeatures2KHR = (PFN_vkGetPhysicalDeviceFeatures2KHR)vk_stub_function_not_loaded;
#endif
      }
      if (!dev->vk.GetPhysicalDeviceProperties2KHR) {
#ifndef NDEBUG
         dev->vk.GetPhysicalDeviceProperties2KHR = (PFN_vkGetPhysicalDeviceProperties2KHR)vk_stub_GetPhysicalDeviceProperties2KHR;
#else
         dev->vk.GetPhysicalDeviceProperties2KHR = (PFN_vkGetPhysicalDeviceProperties2KHR)vk_stub_function_not_loaded;
#endif
      }
      if (!dev->vk.GetPhysicalDeviceFormatProperties2KHR) {
#ifndef NDEBUG
         dev->vk.GetPhysicalDeviceFormatProperties2KHR = (PFN_vkGetPhysicalDeviceFormatProperties2KHR)vk_stub_GetPhysicalDeviceFormatProperties2KHR;
#else
         dev->vk.GetPhysicalDeviceFormatProperties2KHR = (PFN_vkGetPhysicalDeviceFormatProperties2KHR)vk_stub_function_not_loaded;
#endif
      }
      if (!dev->vk.GetPhysicalDeviceImageFormatProperties2KHR) {
#ifndef NDEBUG
         dev->vk.GetPhysicalDeviceImageFormatProperties2KHR = (PFN_vkGetPhysicalDeviceImageFormatProperties2KHR)vk_stub_GetPhysicalDeviceImageFormatProperties2KHR;
#else
         dev->vk.GetPhysicalDeviceImageFormatProperties2KHR = (PFN_vkGetPhysicalDeviceImageFormatProperties2KHR)vk_stub_function_not_loaded;
#endif
      }
      if (!dev->vk.GetPhysicalDeviceQueueFamilyProperties2KHR) {
#ifndef NDEBUG
         dev->vk.GetPhysicalDeviceQueueFamilyProperties2KHR = (PFN_vkGetPhysicalDeviceQueueFamilyProperties2KHR)vk_stub_GetPhysicalDeviceQueueFamilyProperties2KHR;
#else
         dev->vk.GetPhysicalDeviceQueueFamilyProperties2KHR = (PFN_vkGetPhysicalDeviceQueueFamilyProperties2KHR)vk_stub_function_not_loaded;
#endif
      }
      if (!dev->vk.GetPhysicalDeviceMemoryProperties2KHR) {
#ifndef NDEBUG
         dev->vk.GetPhysicalDeviceMemoryProperties2KHR = (PFN_vkGetPhysicalDeviceMemoryProperties2KHR)vk_stub_GetPhysicalDeviceMemoryProperties2KHR;
#else
         dev->vk.GetPhysicalDeviceMemoryProperties2KHR = (PFN_vkGetPhysicalDeviceMemoryProperties2KHR)vk_stub_function_not_loaded;
#endif
      }
      if (!dev->vk.GetPhysicalDeviceSparseImageFormatProperties2KHR) {
#ifndef NDEBUG
         dev->vk.GetPhysicalDeviceSparseImageFormatProperties2KHR = (PFN_vkGetPhysicalDeviceSparseImageFormatProperties2KHR)vk_stub_GetPhysicalDeviceSparseImageFormatProperties2KHR;
#else
         dev->vk.GetPhysicalDeviceSparseImageFormatProperties2KHR = (PFN_vkGetPhysicalDeviceSparseImageFormatProperties2KHR)vk_stub_function_not_loaded;
#endif
      }
   }
   if (dev->instance_info.have_KHR_external_memory_capabilities) {
      if (!dev->vk.GetPhysicalDeviceExternalBufferPropertiesKHR) {
#ifndef NDEBUG
         dev->vk.GetPhysicalDeviceExternalBufferPropertiesKHR = (PFN_vkGetPhysicalDeviceExternalBufferPropertiesKHR)vk_stub_GetPhysicalDeviceExternalBufferPropertiesKHR;
#else
         dev->vk.GetPhysicalDeviceExternalBufferPropertiesKHR = (PFN_vkGetPhysicalDeviceExternalBufferPropertiesKHR)vk_stub_function_not_loaded;
#endif
      }
   }
   if (dev->instance_info.have_KHR_external_semaphore_capabilities) {
      if (!dev->vk.GetPhysicalDeviceExternalSemaphorePropertiesKHR) {
#ifndef NDEBUG
         dev->vk.GetPhysicalDeviceExternalSemaphorePropertiesKHR = (PFN_vkGetPhysicalDeviceExternalSemaphorePropertiesKHR)vk_stub_GetPhysicalDeviceExternalSemaphorePropertiesKHR;
#else
         dev->vk.GetPhysicalDeviceExternalSemaphorePropertiesKHR = (PFN_vkGetPhysicalDeviceExternalSemaphorePropertiesKHR)vk_stub_function_not_loaded;
#endif
      }
   }
   if (dev->instance_info.have_KHR_surface) {
      if (!dev->vk.DestroySurfaceKHR) {
#ifndef NDEBUG
         dev->vk.DestroySurfaceKHR = (PFN_vkDestroySurfaceKHR)vk_stub_DestroySurfaceKHR;
#else
         dev->vk.DestroySurfaceKHR = (PFN_vkDestroySurfaceKHR)vk_stub_function_not_loaded;
#endif
      }
      if (!dev->vk.GetPhysicalDeviceSurfaceSupportKHR) {
#ifndef NDEBUG
         dev->vk.GetPhysicalDeviceSurfaceSupportKHR = (PFN_vkGetPhysicalDeviceSurfaceSupportKHR)vk_stub_GetPhysicalDeviceSurfaceSupportKHR;
#else
         dev->vk.GetPhysicalDeviceSurfaceSupportKHR = (PFN_vkGetPhysicalDeviceSurfaceSupportKHR)vk_stub_function_not_loaded;
#endif
      }
      if (!dev->vk.GetPhysicalDeviceSurfaceCapabilitiesKHR) {
#ifndef NDEBUG
         dev->vk.GetPhysicalDeviceSurfaceCapabilitiesKHR = (PFN_vkGetPhysicalDeviceSurfaceCapabilitiesKHR)vk_stub_GetPhysicalDeviceSurfaceCapabilitiesKHR;
#else
         dev->vk.GetPhysicalDeviceSurfaceCapabilitiesKHR = (PFN_vkGetPhysicalDeviceSurfaceCapabilitiesKHR)vk_stub_function_not_loaded;
#endif
      }
      if (!dev->vk.GetPhysicalDeviceSurfaceFormatsKHR) {
#ifndef NDEBUG
         dev->vk.GetPhysicalDeviceSurfaceFormatsKHR = (PFN_vkGetPhysicalDeviceSurfaceFormatsKHR)vk_stub_GetPhysicalDeviceSurfaceFormatsKHR;
#else
         dev->vk.GetPhysicalDeviceSurfaceFormatsKHR = (PFN_vkGetPhysicalDeviceSurfaceFormatsKHR)vk_stub_function_not_loaded;
#endif
      }
      if (!dev->vk.GetPhysicalDeviceSurfacePresentModesKHR) {
#ifndef NDEBUG
         dev->vk.GetPhysicalDeviceSurfacePresentModesKHR = (PFN_vkGetPhysicalDeviceSurfacePresentModesKHR)vk_stub_GetPhysicalDeviceSurfacePresentModesKHR;
#else
         dev->vk.GetPhysicalDeviceSurfacePresentModesKHR = (PFN_vkGetPhysicalDeviceSurfacePresentModesKHR)vk_stub_function_not_loaded;
#endif
      }
   }
   if (dev->instance_info.have_EXT_headless_surface) {
      if (!dev->vk.CreateHeadlessSurfaceEXT) {
#ifndef NDEBUG
         dev->vk.CreateHeadlessSurfaceEXT = (PFN_vkCreateHeadlessSurfaceEXT)vk_stub_CreateHeadlessSurfaceEXT;
#else
         dev->vk.CreateHeadlessSurfaceEXT = (PFN_vkCreateHeadlessSurfaceEXT)vk_stub_function_not_loaded;
#endif
      }
   }
#ifdef VK_USE_PLATFORM_WAYLAND_KHR
   if (dev->instance_info.have_KHR_wayland_surface) {
      if (!dev->vk.CreateWaylandSurfaceKHR) {
#ifndef NDEBUG
         dev->vk.CreateWaylandSurfaceKHR = (PFN_vkCreateWaylandSurfaceKHR)vk_stub_CreateWaylandSurfaceKHR;
#else
         dev->vk.CreateWaylandSurfaceKHR = (PFN_vkCreateWaylandSurfaceKHR)vk_stub_function_not_loaded;
#endif
      }
      if (!dev->vk.GetPhysicalDeviceWaylandPresentationSupportKHR) {
#ifndef NDEBUG
         dev->vk.GetPhysicalDeviceWaylandPresentationSupportKHR = (PFN_vkGetPhysicalDeviceWaylandPresentationSupportKHR)vk_stub_GetPhysicalDeviceWaylandPresentationSupportKHR;
#else
         dev->vk.GetPhysicalDeviceWaylandPresentationSupportKHR = (PFN_vkGetPhysicalDeviceWaylandPresentationSupportKHR)vk_stub_function_not_loaded;
#endif
      }
   }
#endif
#ifdef VK_USE_PLATFORM_XCB_KHR
   if (dev->instance_info.have_KHR_xcb_surface) {
      if (!dev->vk.CreateXcbSurfaceKHR) {
#ifndef NDEBUG
         dev->vk.CreateXcbSurfaceKHR = (PFN_vkCreateXcbSurfaceKHR)vk_stub_CreateXcbSurfaceKHR;
#else
         dev->vk.CreateXcbSurfaceKHR = (PFN_vkCreateXcbSurfaceKHR)vk_stub_function_not_loaded;
#endif
      }
      if (!dev->vk.GetPhysicalDeviceXcbPresentationSupportKHR) {
#ifndef NDEBUG
         dev->vk.GetPhysicalDeviceXcbPresentationSupportKHR = (PFN_vkGetPhysicalDeviceXcbPresentationSupportKHR)vk_stub_GetPhysicalDeviceXcbPresentationSupportKHR;
#else
         dev->vk.GetPhysicalDeviceXcbPresentationSupportKHR = (PFN_vkGetPhysicalDeviceXcbPresentationSupportKHR)vk_stub_function_not_loaded;
#endif
      }
   }
#endif
#ifdef VK_USE_PLATFORM_WIN32_KHR
   if (dev->instance_info.have_KHR_win32_surface) {
      if (!dev->vk.CreateWin32SurfaceKHR) {
#ifndef NDEBUG
         dev->vk.CreateWin32SurfaceKHR = (PFN_vkCreateWin32SurfaceKHR)vk_stub_CreateWin32SurfaceKHR;
#else
         dev->vk.CreateWin32SurfaceKHR = (PFN_vkCreateWin32SurfaceKHR)vk_stub_function_not_loaded;
#endif
      }
      if (!dev->vk.GetPhysicalDeviceWin32PresentationSupportKHR) {
#ifndef NDEBUG
         dev->vk.GetPhysicalDeviceWin32PresentationSupportKHR = (PFN_vkGetPhysicalDeviceWin32PresentationSupportKHR)vk_stub_GetPhysicalDeviceWin32PresentationSupportKHR;
#else
         dev->vk.GetPhysicalDeviceWin32PresentationSupportKHR = (PFN_vkGetPhysicalDeviceWin32PresentationSupportKHR)vk_stub_function_not_loaded;
#endif
      }
   }
#endif
}

#ifndef NDEBUG
/* generated stub functions */


   
void
vk_stub_CreateDebugUtilsMessengerEXT()
{
   fprintf(stderr, "VK: vkCreateDebugUtilsMessengerEXT is not loaded properly!\n");
   abort();
}
   
void
vk_stub_DestroyDebugUtilsMessengerEXT()
{
   fprintf(stderr, "VK: vkDestroyDebugUtilsMessengerEXT is not loaded properly!\n");
   abort();
}
   
void
vk_stub_SubmitDebugUtilsMessageEXT()
{
   fprintf(stderr, "VK: vkSubmitDebugUtilsMessageEXT is not loaded properly!\n");
   abort();
}
   
void
vk_stub_GetPhysicalDeviceFeatures2KHR()
{
   fprintf(stderr, "VK: vkGetPhysicalDeviceFeatures2KHR is not loaded properly!\n");
   abort();
}
   
void
vk_stub_GetPhysicalDeviceProperties2KHR()
{
   fprintf(stderr, "VK: vkGetPhysicalDeviceProperties2KHR is not loaded properly!\n");
   abort();
}
   
void
vk_stub_GetPhysicalDeviceFormatProperties2KHR()
{
   fprintf(stderr, "VK: vkGetPhysicalDeviceFormatProperties2KHR is not loaded properly!\n");
   abort();
}
   
void
vk_stub_GetPhysicalDeviceImageFormatProperties2KHR()
{
   fprintf(stderr, "VK: vkGetPhysicalDeviceImageFormatProperties2KHR is not loaded properly!\n");
   abort();
}
   
void
vk_stub_GetPhysicalDeviceQueueFamilyProperties2KHR()
{
   fprintf(stderr, "VK: vkGetPhysicalDeviceQueueFamilyProperties2KHR is not loaded properly!\n");
   abort();
}
   
void
vk_stub_GetPhysicalDeviceMemoryProperties2KHR()
{
   fprintf(stderr, "VK: vkGetPhysicalDeviceMemoryProperties2KHR is not loaded properly!\n");
   abort();
}
   
void
vk_stub_GetPhysicalDeviceSparseImageFormatProperties2KHR()
{
   fprintf(stderr, "VK: vkGetPhysicalDeviceSparseImageFormatProperties2KHR is not loaded properly!\n");
   abort();
}
   
void
vk_stub_GetPhysicalDeviceExternalBufferPropertiesKHR()
{
   fprintf(stderr, "VK: vkGetPhysicalDeviceExternalBufferPropertiesKHR is not loaded properly!\n");
   abort();
}
   
void
vk_stub_GetPhysicalDeviceExternalSemaphorePropertiesKHR()
{
   fprintf(stderr, "VK: vkGetPhysicalDeviceExternalSemaphorePropertiesKHR is not loaded properly!\n");
   abort();
}
   
void
vk_stub_DestroySurfaceKHR()
{
   fprintf(stderr, "VK: vkDestroySurfaceKHR is not loaded properly!\n");
   abort();
}
   
void
vk_stub_GetPhysicalDeviceSurfaceSupportKHR()
{
   fprintf(stderr, "VK: vkGetPhysicalDeviceSurfaceSupportKHR is not loaded properly!\n");
   abort();
}
   
void
vk_stub_GetPhysicalDeviceSurfaceCapabilitiesKHR()
{
   fprintf(stderr, "VK: vkGetPhysicalDeviceSurfaceCapabilitiesKHR is not loaded properly!\n");
   abort();
}
   
void
vk_stub_GetPhysicalDeviceSurfaceFormatsKHR()
{
   fprintf(stderr, "VK: vkGetPhysicalDeviceSurfaceFormatsKHR is not loaded properly!\n");
   abort();
}
   
void
vk_stub_GetPhysicalDeviceSurfacePresentModesKHR()
{
   fprintf(stderr, "VK: vkGetPhysicalDeviceSurfacePresentModesKHR is not loaded properly!\n");
   abort();
}
   
void
vk_stub_CreateHeadlessSurfaceEXT()
{
   fprintf(stderr, "VK: vkCreateHeadlessSurfaceEXT is not loaded properly!\n");
   abort();
}
   
#ifdef VK_USE_PLATFORM_WAYLAND_KHR
void
vk_stub_CreateWaylandSurfaceKHR()
{
   fprintf(stderr, "VK: vkCreateWaylandSurfaceKHR is not loaded properly!\n");
   abort();
}
#endif
   
#ifdef VK_USE_PLATFORM_WAYLAND_KHR
void
vk_stub_GetPhysicalDeviceWaylandPresentationSupportKHR()
{
   fprintf(stderr, "VK: vkGetPhysicalDeviceWaylandPresentationSupportKHR is not loaded properly!\n");
   abort();
}
#endif
   
#ifdef VK_USE_PLATFORM_XCB_KHR
void
vk_stub_CreateXcbSurfaceKHR()
{
   fprintf(stderr, "VK: vkCreateXcbSurfaceKHR is not loaded properly!\n");
   abort();
}
#endif
   
#ifdef VK_USE_PLATFORM_XCB_KHR
void
vk_stub_GetPhysicalDeviceXcbPresentationSupportKHR()
{
   fprintf(stderr, "VK: vkGetPhysicalDeviceXcbPresentationSupportKHR is not loaded properly!\n");
   abort();
}
#endif
   
#ifdef VK_USE_PLATFORM_WIN32_KHR
void
vk_stub_CreateWin32SurfaceKHR()
{
   fprintf(stderr, "VK: vkCreateWin32SurfaceKHR is not loaded properly!\n");
   abort();
}
#endif
   
#ifdef VK_USE_PLATFORM_WIN32_KHR
void
vk_stub_GetPhysicalDeviceWin32PresentationSupportKHR()
{
   fprintf(stderr, "VK: vkGetPhysicalDeviceWin32PresentationSupportKHR is not loaded properly!\n");
   abort();
}
#endif

#endif
