#ifndef VK_INSTANCE_H
#define VK_INSTANCE_H

#include <stdbool.h>
#include <vulkan/vulkan.h>

#if defined(__APPLE__)
// Source of MVK_VERSION
#include "MoltenVK/vk_mvk_moltenvk.h"
#endif

struct pipe_screen;
struct vk_device;

struct vk_instance_info {
   uint32_t loader_version;
   bool disable_xcb_surface;

   bool have_EXT_debug_utils;
   bool have_KHR_get_physical_device_properties2;
   bool have_KHR_external_memory_capabilities;
   bool have_KHR_external_semaphore_capabilities;
   bool have_MVK_moltenvk;
   bool have_KHR_surface;
   bool have_EXT_headless_surface;
   bool have_KHR_wayland_surface;
   bool have_KHR_xcb_surface;
   bool have_KHR_win32_surface;

};

bool
vk_create_instance(struct vk_device *dev);

void
vk_verify_instance_extensions(struct vk_device *dev);

/* stub functions that get inserted into the dispatch table if they are not
 * properly loaded.
 */
void vk_stub_CreateDebugUtilsMessengerEXT(void);
void vk_stub_DestroyDebugUtilsMessengerEXT(void);
void vk_stub_SubmitDebugUtilsMessageEXT(void);
void vk_stub_GetPhysicalDeviceFeatures2KHR(void);
void vk_stub_GetPhysicalDeviceProperties2KHR(void);
void vk_stub_GetPhysicalDeviceFormatProperties2KHR(void);
void vk_stub_GetPhysicalDeviceImageFormatProperties2KHR(void);
void vk_stub_GetPhysicalDeviceQueueFamilyProperties2KHR(void);
void vk_stub_GetPhysicalDeviceMemoryProperties2KHR(void);
void vk_stub_GetPhysicalDeviceSparseImageFormatProperties2KHR(void);
void vk_stub_GetPhysicalDeviceExternalBufferPropertiesKHR(void);
void vk_stub_GetPhysicalDeviceExternalSemaphorePropertiesKHR(void);
void vk_stub_DestroySurfaceKHR(void);
void vk_stub_GetPhysicalDeviceSurfaceSupportKHR(void);
void vk_stub_GetPhysicalDeviceSurfaceCapabilitiesKHR(void);
void vk_stub_GetPhysicalDeviceSurfaceFormatsKHR(void);
void vk_stub_GetPhysicalDeviceSurfacePresentModesKHR(void);
void vk_stub_CreateHeadlessSurfaceEXT(void);
void vk_stub_CreateWaylandSurfaceKHR(void);
void vk_stub_GetPhysicalDeviceWaylandPresentationSupportKHR(void);
void vk_stub_CreateXcbSurfaceKHR(void);
void vk_stub_GetPhysicalDeviceXcbPresentationSupportKHR(void);
void vk_stub_CreateWin32SurfaceKHR(void);
void vk_stub_GetPhysicalDeviceWin32PresentationSupportKHR(void);

struct pipe_screen;
struct pipe_resource;

#endif
